import { _ as _applyDecoratedDescriptor, a as _initializerDefineProperty, b as _defineProperty } from '../_rollupPluginBabelHelpers-e795903d.js';
import { buildTask } from 'ember-concurrency/-private/async-arrow-runtime';
import { getOwner } from '@ember/application';
import Service from '@ember/service';
import 'ember-concurrency';
import { tracked } from '@glimmer/tracking';

var _class, _descriptor;
let ShikiService = (_class = class ShikiService extends Service {
  constructor(...args) {
    super(...args);
    _initializerDefineProperty(this, "isInitialized", _descriptor, this);
    _defineProperty(this, "highlighter", void 0);
    _defineProperty(this, "BUNDLED_LANGUAGES", void 0);
    _defineProperty(this, "BUNDLED_THEMES", void 0);
    _defineProperty(this, "loadLanguageTask", buildTask(() => ({
      context: this,
      generator: function* (language) {
        if (!this.highlighter || !this.BUNDLED_LANGUAGES) {
          return;
        }
        if (!this.highlighter.getLoadedLanguages().includes(language)) {
          // Check if the language is supported by Shiki
          const bundles = this.BUNDLED_LANGUAGES.filter(bundle => {
            // Languages are specified by their id, they can also have aliases (i. e. "js" and "javascript")
            return bundle.id === language || bundle.aliases?.includes(language);
          });
          if (bundles.length > 0) {
            yield this.highlighter.loadLanguage(language);
          }
        }
      }
    }), {
      enqueue: true
    }, "loadLanguageTask", null));
    _defineProperty(this, "loadThemeTask", buildTask(() => ({
      context: this,
      generator: function* (theme) {
        if (!this.highlighter || !this.BUNDLED_THEMES) {
          return;
        }
        if (!this.highlighter.getLoadedThemes().includes(theme)) {
          // Check if the theme is supported by Shiki
          if (this.BUNDLED_THEMES.includes(theme)) {
            yield this.highlighter.loadTheme(theme);
          }
        }
      }
    }), {
      enqueue: true
    }, "loadThemeTask", null));
    _defineProperty(this, "initialize", buildTask(() => ({
      context: this,
      generator: function* () {
        if (this.isInitialized) {
          return;
        }

        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const config = getOwner(this)?.resolveRegistration('config:environment');
        const {
          defaultLanguages,
          defaultThemes
        } = config['ember-shiki'] ?? {};
        const {
          BUNDLED_LANGUAGES,
          BUNDLED_THEMES,
          getHighlighter,
          setCDN,
          setWasm
        } = yield import('shiki');

        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const fastboot = getOwner(this)?.lookup('service:fastboot');
        if (fastboot && fastboot.isFastBoot) {
          const wasmResponse = yield fetch(`${this.cdnUrl}dist/onig.wasm`).then(
          // eslint-disable-next-line @typescript-eslint/no-explicit-any
          res => res.arrayBuffer());
          setWasm(wasmResponse);
        }
        setCDN(this.cdnUrl);
        this.BUNDLED_LANGUAGES = BUNDLED_LANGUAGES;
        this.BUNDLED_THEMES = BUNDLED_THEMES;
        this.highlighter = yield getHighlighter({
          themes: defaultThemes,
          langs: defaultLanguages
        });
        this.isInitialized = true;
      }
    }), {
      enqueue: true
    }, "initialize", null));
    _defineProperty(this, "render", buildTask(() => ({
      context: this,
      generator: function* (code, language, theme, lineHighlights) {
        if (!this.isInitialized) {
          yield this.initialize.perform();
        }
        if (language) {
          yield this.loadLanguageTask.perform(language);
        }
        if (theme) {
          yield this.loadThemeTask.perform(theme);
        }
        const shikiConfig = {
          lang: language,
          theme: theme,
          lineOptions: lineHighlights ? this.mapLineHighlightsToShikiOptions(lineHighlights) : undefined
        };
        return this.highlighter?.codeToHtml(code, shikiConfig) ?? '';
      }
    }), {}, "render", null));
  }
  get cdnUrl() {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const config = getOwner(this)?.resolveRegistration('config:environment');
    const {
      cdnUrl
    } = config['ember-shiki'] ?? {};
    if (cdnUrl) {
      return cdnUrl;
    }
    const SHIKI_VERSION = '0.14.3';
    return `https://cdn.jsdelivr.net/npm/shiki@${SHIKI_VERSION}/`;
  }
  mapLineHighlightsToShikiOptions(lineHighlights) {
    const options = [];
    const optionToCssClass = {
      neutral: 'line--highlight-neutral',
      add: 'line--highlight-add',
      remove: 'line--highlight-remove'
    };
    for (const lineOption of lineHighlights) {
      const end = lineOption.end ?? lineOption.start;
      for (let line = lineOption.start; line <= end; line += 1) {
        options.push({
          line,
          classes: [optionToCssClass[lineOption.type]]
        });
      }
    }
    return options;
  }
}, (_descriptor = _applyDecoratedDescriptor(_class.prototype, "isInitialized", [tracked], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: function () {
    return false;
  }
})), _class);
// this service using `Owner.lookup('service:shiki')`, as well
// as to check when you pass the service name as an argument to the decorator,
// like `@service('shiki') declare altName: ShikiService;`.

export { ShikiService as default };
//# sourceMappingURL=shiki.js.map
